package negocios;

import camadaDados.Conexao;
import camadaDados.RepositorioAbastece;
import camadaDados.RepositorioAlerta;
import camadaDados.RepositorioPosto;
import camadaDados.RepositorioVeiculo;
import classesBasicas.*;
import exceptions.*;
import java.sql.SQLException;
import java.util.ArrayList;

public class ControleVeiculo {
	
        final double TAXA_CONSUMO = 1.2;
        final int PRAZO_TROCA_OLEO = 180;
	Veiculo veiculo;
        Conexao cnn;
        RepositorioVeiculo repositorioVeiculo;
        RepositorioAbastece repositorioAbastece;
        RepositorioPosto repositorioPosto;
        RepositorioAlerta repositorioAlerta;
        
        public ControleVeiculo(Conexao cnn){
            this.cnn = cnn;
            repositorioAbastece = new RepositorioAbastece(cnn);
            repositorioVeiculo = new RepositorioVeiculo(cnn);
            repositorioAlerta = new RepositorioAlerta(cnn);
            repositorioPosto = new RepositorioPosto(cnn);
        }
	
	public void cadastrarVeiculo(String placa, String chassi, int ano, String modelo,
                    String cor, double quilometragem, String tipoCombustivel,
                    double valorCompra, double valorSeguro, String vencimentoSeguro,
                    String dataCompra) throws PlacaInvalida, ChassiInvalido, ModeloInvalido, CorInvalida, CombustivelInvalido, PlacaJaCadastrada{
                placa.toUpperCase();
		if(placa.length() != 7){
                    throw new PlacaInvalida();
		}
		
                String pre = placa.substring(0,3);
                for(int i=0;i<pre.length();i++){
                    if(pre.charAt(i) < 'A' || pre.charAt(i) > 'Z'){
                        throw new PlacaInvalida();
                    }
                }	
		
                try{
                Integer.parseInt(placa.substring(3));
                }
                catch(Exception e){
                    throw new PlacaInvalida();
                }
			
		if(chassi.length() != 20){
			throw new ChassiInvalido();
		}
		if(cor.length() == 0){
                        throw new ModeloInvalido();			
		}
		if(cor.length() == 0){
                        throw new CorInvalida();			
		}
		if(tipoCombustivel.length() == 0){
                        throw new CombustivelInvalido();
			
		}
		veiculo = new Veiculo(placa, chassi, ano, modelo, cor, quilometragem, tipoCombustivel, valorCompra, valorSeguro, vencimentoSeguro, "01/01/0001", "01/01/0001", dataCompra);
                
                try{
                    repositorioVeiculo.inserirVeiculo(veiculo);
			//inserir no banco;
		}catch(Exception e){
                        throw new PlacaJaCadastrada();
		}
	}
	
	public void buscarVeiculo(String placa) throws PlacaInvalida, VeiculoNaoEncontrado, NaoEncontrado, SQLException{
            if(placa.length() != 7){
                throw new PlacaInvalida();
            }
            placa.toUpperCase();
            String pre = placa.substring(0,3);
            for(int i=0;i<pre.length();i++){
                if(pre.charAt(i) < 'A' || pre.charAt(i) > 'Z'){
                    throw new PlacaInvalida();
		}
            }	
            
            try{
                Integer.parseInt(placa.substring(3));
            }
            catch(Exception e){
                throw new PlacaInvalida();
            }
            try{
                //veiculo = buscar veiculo no banco
                veiculo = repositorioVeiculo.buscarVeiculo(placa);
            }
            catch(SQLException e){
                throw new VeiculoNaoEncontrado();
            }
	}
        
        public double analisarConsumo(String placa) throws PlacaInvalida, VeiculoNaoEncontrado, SQLException, AbastecimentoInexistente, NaoEncontrado{
            buscarVeiculo(placa);
            int litros = repositorioAbastece.buscarQuantidadesLitrosAbastecidos(placa);
            Abastece ab = repositorioAbastece.buscarUltimoAbastecimento(placa);
            double qtd = ab.getQuilometragem() - veiculo.getQuilometragem();
            qtd /= litros;
            return qtd;
        }
        
        public void realizarAbatecimento(int cep, int numero, String nome, int reputacao, String rua, String bairro, 
                String cidade, String estado, String pais, String latitude, String longitude,
                String placa, int quilometragem, String dataHora, String combustivel, int litros, int custo) throws AbasteciementoJahInserido, PostoJahExiste ,PlacaInvalida, VeiculoNaoEncontrado, SQLException, NaoEncontrado, AbastecimentoInexistente, PostoInexistente, AlertaJahExistente{
                Posto posto = new Posto(cep, numero, nome, reputacao, rua, bairro, cidade, estado, pais, latitude, longitude);
            try{
                posto = repositorioPosto.buscarPosto(cep, numero);
               // buscarPosto(posto); ver com danilo
            }catch(Exception e){
                repositorioPosto.cadastrarPosto(posto);
               // inserirPosto(posto); danilo;
            }
            try{
                Abastece abastece = repositorioAbastece.buscarUltimoAbastecimento(placa);
                int reput = (quilometragem - abastece.getQuilometragem())/abastece.getCusto();
                posto.setReputacao(reput);   //atualizar reputacao do posto;
                repositorioPosto.atualizarPosto(posto);
                //atualizarPosto(posto);
                double consumoMedio = analisarConsumo(placa);
                double consumoAtual = (quilometragem - abastece.getQuilometragem())/abastece.getLitros(); //dependente de danilo
                if(consumoAtual*TAXA_CONSUMO > consumoMedio){
                    Alerta alerta = new Alerta(cep, placa, "Consumo excessivo detectado", dataHora, dataHora, 0);
                    repositorioAlerta.inserirAlerta(alerta);
                    // inserirAlerta(alerta);          Depende Danilo
                }
            }catch(AbastecimentoInexistente e){
                System.out.println("abastecimento inexistente");
            }
            buscarVeiculo(placa);
            System.out.println("1");
            String data = veiculo.getData_troca_oleo();
            System.out.println("2"+data);
            int valor = Integer.parseInt(data.substring(0, 2)) + Integer.parseInt(data.substring(3, 5))*30 + Integer.parseInt(data.substring(6, 10))*360 ;
            System.out.println("3"+dataHora);
            int valor2 = Integer.parseInt(dataHora.substring(0, 2)) + Integer.parseInt(dataHora.substring(3, 5))*30 + Integer.parseInt(dataHora.substring(6, 10))*360 ;
            System.out.println("4");
            if(valor+PRAZO_TROCA_OLEO > valor2){
                Alerta alerta = new Alerta(cep, placa, "Troca de oleo deve ser realizada", dataHora, dataHora, 0);
                repositorioAlerta.inserirAlerta(alerta);
                // inserirAlerta(alerta);          Depende Danilo
            }
            System.out.println("5");
            Abastece abastece = new Abastece(placa, cep, numero,  quilometragem, dataHora, combustivel, litros, custo);
            System.out.println("6");
            try{
                repositorioAbastece.inserirAbastecimento(abastece);
            }catch(AbasteciementoJahInserido e){
                System.out.println("ja inserido");
            }
            System.out.println("7");
          //  inserirAbastecimento(abastece);   danilo
        }
        
        public int analisarCustoVeiculo(String placa) throws NaoEncontrado{
            try{    
                return repositorioVeiculo.calcularCusto(placa);
            }catch(Exception e){
                throw new NaoEncontrado();
            }
        }
        
        
        public float distFrom(float lat1, float lng1, float lat2, float lng2) {
            double earthRadius = 3958.75;
            double dLat = Math.toRadians(lat2-lat1);
            double dLng = Math.toRadians(lng2-lng1);
            double a = Math.sin(dLat/2) * Math.sin(dLat/2) +
                       Math.cos(Math.toRadians(lat1)) * Math.cos(Math.toRadians(lat2)) *
                       Math.sin(dLng/2) * Math.sin(dLng/2);
            double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
            double dist = earthRadius * c;

            int meterConversion = 1609;

            return new Float(dist * meterConversion).floatValue();
         }
        
        public String indicarLocalAbastecimento(String latitude, String longitude) throws PostoInexistente{
            float lat1 = Float.parseFloat(latitude), lng1 = Float.parseFloat(longitude);
            ArrayList<Posto> postos = new ArrayList<Posto>(10);//METODO DE DANILO PARA PEGAR TODOS OS POSTOS
            int melhor = -1;
            int lastRep = Integer.MAX_VALUE;
            for(int i = 0; i<postos.size(); i++){
                Posto temp = postos.get(i);
                float lat2 = Float.parseFloat(temp.getLatitude());
                float lng2 = Float.parseFloat(temp.getLongitude());
                float dist = distFrom(lat1, lng1, lat2, lng2);
                if (dist <= 300000000){
                    if (temp.getReputacao() < lastRep){
                        melhor = i;
                        lastRep = temp.getReputacao();
                    }   
                }
            }
            if (melhor >= 0){
                Posto p = postos.get(melhor);
                String retorno = p.getCep()+" "+p.getNumero()+" "+p.getNome()+" "+p.getReputacao()+" "+p.getRua()+" "+p.getBairro()+" "+p.getCidade()+" "+p.getEstado()+" "+p.getPais()+" "+p.getLatitude()+" "+p.getLongitude();
                return retorno;
            }
            
            throw new PostoInexistente();
        }
        
	public void atualizarVeiculo(String placa, String chassi, int ano, String modelo,
                    String cor, double quilometragem, String tipoCombustivel,
                    double valorCompra, double valorSeguro, String vencimentoSeguro,
                    String dataTrocaOleo, String dataCompra) throws PlacaInvalida, ChassiInvalido, CombustivelInvalido{
		boolean fail = false;
                placa.toUpperCase();
		if(placa.length() != 7){
                    throw new PlacaInvalida();
		}
		
                String pre = placa.substring(0,3);
                for(int i=0;i<pre.length();i++){
                    if(pre.charAt(i) < 'A' || pre.charAt(i) > 'Z'){
                        throw new PlacaInvalida();
                    }
                }
                
                try
                {
                    Integer.parseInt(placa.substring(3));
                }
                catch(Exception e)
                {
                    throw new PlacaInvalida();
                }
		
		
		if(chassi.length() != 20)
                {
                        throw new ChassiInvalido();
		}
                
		if(tipoCombustivel.length() == 0)
                {
                        throw new CombustivelInvalido();
		}
		veiculo = new Veiculo(placa, chassi, ano, modelo, cor, quilometragem, tipoCombustivel,valorCompra, valorSeguro, vencimentoSeguro,  dataTrocaOleo, "01/01/0001", dataCompra);
                
                //atualizar veiculo no banco
                try{
                    repositorioVeiculo.atualizarVeiculo(veiculo);
                }
                catch(SQLException e){
                    e.printStackTrace();
                }
	}
	
	public void removerVeiculo(String placa) throws VeiculoJaRemovido, SQLException{
            repositorioVeiculo.descadastrarVeiculo(placa);	
	}
	
	public Veiculo getVeiculo(){
		return veiculo;
	}
        

}